@@ -1,5 +1,5 @@
 /* Tcl/Tk command definitions for Insight - Stack.
-   Copyright 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "defs.h"
 #include "target.h"
@@ -24,6 +24,7 @@
 #include "linespec.h"
 #include "block.h"
 #include "dictionary.h"
+#include "varobj.h"
 
 #include <tcl.h>
 #include "gdbtk.h"
@@ -97,13 +98,13 @@ gdb_block_vars (ClientData clientData, Tcl_Interp *interp,
     }
 
   Tcl_SetListObj (result_ptr->obj_ptr, 0, NULL);
-  if (deprecated_selected_frame == NULL)
+  if (!target_has_registers)
     return TCL_OK;
 
   start = string_to_core_addr (Tcl_GetStringFromObj (objv[1], NULL));
   end   = string_to_core_addr (Tcl_GetStringFromObj (objv[2], NULL));
   
-  block = get_frame_block (deprecated_selected_frame, 0);
+  block = get_frame_block (get_selected_frame (NULL), 0);
 
   while (block != 0)
     {
@@ -168,10 +169,12 @@ gdb_get_blocks (ClientData clientData, Tcl_Interp *interp,
 
   Tcl_SetListObj (result_ptr->obj_ptr, 0, NULL);
   
-  if (deprecated_selected_frame != NULL)
+  if (target_has_registers)
     {
-      block = get_frame_block (deprecated_selected_frame, 0);
-      pc = get_frame_pc (deprecated_selected_frame);
+      struct frame_info *frame = get_selected_frame (NULL);
+
+      block = get_frame_block (frame, 0);
+      pc = get_frame_pc (frame);
       while (block != 0)
 	{
 	  junk = 0;
@@ -292,7 +295,7 @@ gdb_get_vars_command (ClientData clientData, Tcl_Interp *interp,
       return TCL_ERROR;
     }
 
-  arguments = (int) clientData;
+  arguments = *(int*) clientData;
 
   /* Initialize the result pointer to an empty list. */
 
@@ -317,10 +320,10 @@ gdb_get_vars_command (ClientData clientData, Tcl_Interp *interp,
   else
     {
       /* Specified currently selected frame */
-      if (deprecated_selected_frame == NULL)
+      if (!target_has_registers)
 	return TCL_OK;
 
-      block = get_frame_block (deprecated_selected_frame, 0);
+      block = get_frame_block (get_selected_frame (NULL), 0);
     }
 
   while (block != 0)
@@ -387,7 +390,7 @@ gdb_selected_block (ClientData clientData, Tcl_Interp *interp,
   char *start = NULL;
   char *end   = NULL;
 
-  if (deprecated_selected_frame == NULL)
+  if (!target_has_registers)
     {
       xasprintf (&start, "%s", "");
       xasprintf (&end, "%s", "");
@@ -395,7 +398,7 @@ gdb_selected_block (ClientData clientData, Tcl_Interp *interp,
   else
     {
       struct block *block;
-      block = get_frame_block (deprecated_selected_frame, 0);
+      block = get_frame_block (get_selected_frame (NULL), 0);
       xasprintf (&start, "0x%s", paddr_nz (BLOCK_START (block)));
       xasprintf (&end, "0x%s", paddr_nz (BLOCK_END (block)));
     }
@@ -427,13 +430,13 @@ gdb_selected_frame (ClientData clientData, Tcl_Interp *interp,
 {
   char *frame;
 
-  if (deprecated_selected_frame == NULL)
+  if (!target_has_registers)
     xasprintf (&frame, "%s","");
   else
     /* FIXME: cagney/2002-11-19: This should be using get_frame_id()
        to identify the frame and *NOT* get_frame_base().  */
     xasprintf (&frame, "0x%s",
-	       paddr_nz (get_frame_base (deprecated_selected_frame)));
+	       paddr_nz (get_frame_base (get_selected_frame (NULL))));
 
   Tcl_SetStringObj (result_ptr->obj_ptr, frame, -1);
 
@@ -535,13 +538,13 @@ get_frame_name (Tcl_Interp *interp, Tcl_Obj *list, struct frame_info *fi)
 
   if (get_frame_type (fi) == DUMMY_FRAME)
     {
-      objv[0] = Tcl_NewStringObj ("<function called from gdb>\n", -1);
+      objv[0] = Tcl_NewStringObj ("<function called from gdb>", -1);
       Tcl_ListObjAppendElement (interp, list, objv[0]);
       return;
     }
   if ((get_frame_type (fi) == SIGTRAMP_FRAME))
     {
-      objv[0] = Tcl_NewStringObj ("<signal handler called>\n", -1);
+      objv[0] = Tcl_NewStringObj ("<signal handler called>", -1);
       Tcl_ListObjAppendElement (interp, list, objv[0]);
       return;
     }
@@ -598,16 +601,6 @@ get_frame_name (Tcl_Interp *interp, Tcl_Obj *list, struct frame_info *fi)
 			      DMGL_ANSI);
 #endif
       objv[0] = Tcl_NewStringObj (funname != NULL ? funname : "??", -1);
-#ifdef PC_LOAD_SEGMENT
-      /* If we couldn't print out function name but if can figure out what
-         load segment this pc value is from, at least print out some info
-         about its load segment. */
-      if (!funname)
-	{
-	  Tcl_AppendStringsToObj (objv[0], " from ", PC_LOAD_SEGMENT (fi->pc),
-				  (char *) NULL);
-	}
-#endif
 #ifdef PC_SOLIB
       if (!funname)
 	{
