@@ -1,5 +1,5 @@
 # Interface between GDB and Insight.
-# Copyright 1997, 1998, 1999, 2001, 2002 Red Hat, Inc.
+# Copyright (C) 1997, 1998, 1999, 2001, 2002, 2004, 2008 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License (GPL) as published by
@@ -119,6 +119,7 @@ proc gdbtk_tcl_preloop { } {
     set current_args [gdb_get_inferior_args]
     set current_dir $gdb_current_directory
     Session::notice_file_change
+    set_baud
     if {[string length $current_args] > 0} {
       gdb_set_inferior_args $current_args
       gdb_cmd "cd $current_dir"
@@ -255,10 +256,25 @@ proc gdbtk_quit_check {} {
 # ------------------------------------------------------------------
 proc gdbtk_quit {} {
   if {[gdbtk_quit_check]} {
-    gdb_force_quit
+    gdbtk_force_quit
   }
 }
 
+# ------------------------------------------------------------------
+#  PROCEDURE:  gdbtk_force_quit - Quit the debugger immediately
+# ------------------------------------------------------------------
+proc gdbtk_force_quit {} {
+  # If we don't delete source windows, GDB hooks will
+  # try to update them as we exit
+  foreach win [ManagedWin::find SrcWin] {
+    delete object $win
+  }
+  # Calling gdb_force_quit is probably not necessary here
+  # because it should have been called when the source window(s)
+  # were deleted, but just in case...
+  gdb_force_quit
+}
+
 # ------------------------------------------------------------------
 #  PROCEDURE:  gdbtk_cleanup - called by GDB immediately
 #         before exiting.  Last chance to cleanup!
@@ -285,7 +301,7 @@ proc gdbtk_cleanup {} {
 # PROC: gdbtk_tcl_query -
 # ------------------------------------------------------------------
 proc gdbtk_tcl_query {message {default yes}} {
-  global gdb_checking_for_exit gdbtk_state tcl_platform
+  global gdb_checking_for_exit gdbtk_state gdbtk_platform
 
   # FIXME We really want a Help button here.  But Tk's brain-damaged
   # modal dialogs won't really allow it.  Should have async dialog
@@ -301,7 +317,7 @@ proc gdbtk_tcl_query {message {default yes}} {
     set modal "system"
   }
   
-  if {$tcl_platform(platform) == "windows"} {
+  if {$gdbtk_platform(platform) == "windows"} {
     # On Windows, we want to only ask each question once.
     # If we're already asking the question, just wait for the answer
     # to come back.
@@ -331,7 +347,7 @@ proc gdbtk_tcl_query {message {default yes}} {
     # question.
     set r [tk_messageBox -icon warning -default $default \
 	     -message $message -title $title \
-	     -type yesno -modal $modal -parent .]
+	     -type yesno -parent .]
   }
 
   update idletasks
@@ -349,6 +365,9 @@ proc gdbtk_tcl_warning {message} {
 
   switch -regexp $message {
         "Unable to find dynamic linker breakpoint function.*" {return}
+	"Internal error.*" { gdbtk_tcl_fputs_error $message }
+        "incomplete CFI.*" { gdbtk_tcl_fputs_error $message }
+	"RTTI symbol not found for class.*" { gdbtk_tcl_fputs_error $message }
         default {show_warning $message}
        }
 }
@@ -357,7 +376,7 @@ proc gdbtk_tcl_warning {message} {
 # PROC: show_warning -
 # ------------------------------------------------------------------
 proc show_warning {message} {
-  global tcl_platform
+  global gdbtk_platform
 
   # FIXME We really want a Help button here.  But Tk's brain-damaged
   # modal dialogs won't really allow it.  Should have async dialog
@@ -376,14 +395,14 @@ proc show_warning {message} {
 # another thread is not required.
 
  
-  if {$tcl_platform(platform) == "windows"} {
+  if {$gdbtk_platform(platform) == "windows"} {
       ide_messageBox [list set r] -icon warning \
         -default ok -message $message -title $title \
         -type ok -modal $modal -parent .
   } else {
     set r [tk_messageBox -icon warning -default ok \
              -message $message -title $title \
-             -type ok -modal $modal -parent .]
+             -type ok -parent .]
   }
 } 
 
@@ -403,7 +422,7 @@ proc gdbtk_tcl_fputs {message} {
   # Restore the fputs hook, in case anyone forgot to put it back...
   gdb_restore_fputs
 
-  if {$gdbtk_state(console) != ""} {
+  if {[info exists gdbtk_state(console)] &&   $gdbtk_state(console) != ""} {
     $gdbtk_state(console) insert $message
   }
 }
@@ -419,7 +438,7 @@ proc echo {args} {
 # PROC: gdbtk_tcl_fputs_error - write an error message
 # ------------------------------------------------------------------
 proc gdbtk_tcl_fputs_error {message} {
-  if {$::gdbtk_state(console) != ""} {
+  if {[info exists ::gdbtk_state(console)] && $::gdbtk_state(console) != ""} {
     $::gdbtk_state(console) insert $message err_tag
     update
   }
@@ -429,7 +448,7 @@ proc gdbtk_tcl_fputs_error {message} {
 # PROC: gdbtk_tcl_fputs_log - write a log message
 # ------------------------------------------------------------------
 proc gdbtk_tcl_fputs_log {message} {
-  if {$::gdbtk_state(console) != ""} {
+  if {[info exists ::gdbtk_state(console)] && $::gdbtk_state(console) != ""} {
     $::gdbtk_state(console) insert $message log_tag
     update
   }
@@ -439,10 +458,22 @@ proc gdbtk_tcl_fputs_log {message} {
 # PROC: gdbtk_tcl_fputs_target - write target output
 # ------------------------------------------------------------------
 proc gdbtk_tcl_fputs_target {message} {
-  if {$::gdbtk_state(console) != ""} {
-    $::gdbtk_state(console) insert $message target_tag
-    update
+  if {$::gdbtk_state(console) == ""} {
+    ManagedWin::open Console -force
   }
+  $::gdbtk_state(console) insert $message target_tag
+  update
+}
+
+
+# ------------------------------------------------------------------
+# PROC: gdbtk_tcl_fputs_target_err - write target error output
+# ------------------------------------------------------------------
+proc gdbtk_tcl_fputs_target_err {message} {
+  if {$::gdbtk_state(console) == ""} {
+    ManagedWin::open Console -force
+  }  
+  $::gdbtk_state(console) insert $message err_tag
 }
 
 # ------------------------------------------------------------------
@@ -512,7 +543,7 @@ proc gdbtk_tcl_trace_find_hook {arg from_tty} {
 # ------------------------------------------------------------------
 proc gdb_run_readline_command {command args} {
   global gdbtk_state
-#  debug "$command $args"
+  debug "$command $args"
   set gdbtk_state(readlineArgs) $args
   set gdbtk_state(readlineShowUser) 1
   gdb_cmd $command
@@ -524,7 +555,7 @@ proc gdb_run_readline_command {command args} {
 # ------------------------------------------------------------------
 proc gdb_run_readline_command_no_output {command args} {
   global gdbtk_state
-#  debug "$command $args"
+  debug "$command $args"
   set gdbtk_state(readlineArgs) $args
   set gdbtk_state(readlineShowUser) 0
   gdb_cmd $command
@@ -535,7 +566,7 @@ proc gdb_run_readline_command_no_output {command args} {
 # ------------------------------------------------------------------
 proc gdbtk_tcl_readline_begin {message} {
   global gdbtk_state
-#  debug "readline begin"
+#  debug
   set gdbtk_state(readline) 0
   if {$gdbtk_state(console) != "" && $gdbtk_state(readlineShowUser)} {
     $gdbtk_state(console) insert $message
@@ -547,12 +578,12 @@ proc gdbtk_tcl_readline_begin {message} {
 # ------------------------------------------------------------------
 proc gdbtk_tcl_readline {prompt} {
   global gdbtk_state
-#  debug "$prompt"
+#  debug "prompt=$prompt"
   if {[info exists gdbtk_state(readlineArgs)]} {
     # Not interactive, so pop the list, and print element.
     set cmd [lvarpop gdbtk_state(readlineArgs)]
-    if {$gdbtk_state(readlineShowUser)} {
-      command::insert_command $cmd
+    if {$gdbtk_state(console) != "" && $gdbtk_state(readlineShowUser)} {
+      $gdbtk_state(console) insert $cmd
     }
   } else {
     # Interactive.
@@ -573,10 +604,9 @@ proc gdbtk_tcl_readline {prompt} {
 # ------------------------------------------------------------------
 proc gdbtk_tcl_readline_end {} {
   global gdbtk_state
-#  debug "readline_end"
+#  debug
   catch {unset gdbtk_state(readlineArgs)}
-  unset gdbtk_state(readlineActive)
-  command::end_multi_line_input
+  catch {unset gdbtk_state(readlineActive)}
 }
 
 # ------------------------------------------------------------------
@@ -675,10 +705,10 @@ proc gdbtk_memory_changed {} {
 #         - file_changed_hook                                      #
 #            Called in file_command. The tcl hook is               #
 #            "gdbtk_tcl_file_changed"                              #
-#         - pre_add_symbol_hook                                    #
+#         - deprecated_pre_add_symbol_hook                         #
 #            Called in symbol_file_add before loading. The tcl     #
 #            hook is "gdbtk_tcl_pre_add_symbol"                    #
-#         - post_add_symbol_hook                                   #
+#         - deprecated_post_add_symbol_hook                        #
 #            Called in symbol_file_add when finished loading       #
 #            a symbol file. The tcl hook is                        #
 #            "gdbtk_tcl_post_add_symbol"                           #
@@ -797,29 +827,35 @@ proc gdbtk_tcl_exec_file_display {filename} {
 #  6: shared library name if the pc is in a shared lib
 #
 # ------------------------------------------------------------------
-proc gdbtk_locate_main {} {
-  set result {}
-  set main_names [pref get gdb/main_names]
-  debug "Searching $main_names"
+proc gdbtk_locate_main {{init ""}} {
+  global _main_cache gdb_exe_name
+  debug
 
+  if {$init == "" && $_main_cache != ""} {
+    #debug "returning $_main_cache from cache"
+    return $_main_cache
+  }
+  set _main_cache {}
+
+  set main_names [pref get gdb/main_names]
   foreach main $main_names {
-    if {![catch {gdb_search functions $main -static 1}] \
-        && ![catch {gdb_loc $main} linespec]} {
-      set result $linespec
+    if {![catch {gdb_loc $main} linespec]} {
+      set _main_cache $linespec
       break
     }
   }
-  if {$result == {} 
+  if {$_main_cache == {} 
       && ![catch gdb_entry_point entry_point]
       && ![catch {gdb_loc "*$entry_point"} linespec]} {
-    set result $linespec
+    set _main_cache $linespec
   }
   
   # need to see if result is valid
-  lassign $result file func ffile line addr rest
-  if {$addr == 0x0 && $func == {}} { set result {} }
+  lassign $_main_cache file func ffile line addr rest
+  if {$addr == 0x0 && $func == {}} { set _main_cache {} }
 
-  return $result
+  #debug "returning $_main_cache"
+  return $_main_cache
 }
 
 ##############################################
@@ -863,7 +899,7 @@ proc set_exe {} {
       return
     } elseif {[string match {*no debugging symbols found*} $msg]} {
       tk_messageBox -icon error -default ok \
-	-title "GDB" -type ok -modal system \
+	-title "GDB" -type ok \
 	-message "This executable has no debugging information."
     }
 
@@ -880,7 +916,7 @@ proc set_exe {} {
 # ------------------------------------------------------------------
 
 proc _open_file {{file ""}} {
-  global gdb_running gdb_downloading tcl_platform
+  global gdb_running gdb_downloading gdbtk_platform
   
   if {$gdb_running || $gdb_downloading} {
     # We are already running/downloading something..
@@ -926,7 +962,7 @@ proc _open_file {{file ""}} {
   }
   # Add the base dir for this file to the source search path.
   set root [file dirname $file]
-  if {$tcl_platform(platform) == "windows"} {
+  if {$gdbtk_platform(os) == "cygwin"} {
     set root [ide_cygwin_path to_posix $root]
     set file [ide_cygwin_path to_posix $file]
   }
@@ -1061,8 +1097,8 @@ proc set_target_name {{prompt 1}} {
 # ------------------------------------------------------------------
 proc set_target {} {
   global gdb_target_cmd gdb_target_changed gdb_pretty_name gdb_target_name
-#  debug "gdb_target_changed=$gdb_target_changed gdb_target_cmd=\"$gdb_target_cmd\""
-#  debug "gdb_target_name=$gdb_target_name"
+  #debug "gdb_target_changed=$gdb_target_changed gdb_target_cmd=\"$gdb_target_cmd\""
+  #debug "gdb_target_name=$gdb_target_name"
   if {$gdb_target_cmd == "" && ![TargetSelection::native_debugging]} {
     if {$gdb_target_name == ""} {
       set prompt 1
@@ -1085,8 +1121,10 @@ proc set_target {} {
     update
     catch {gdb_cmd "detach"}
     debug "CONNECTING TO TARGET: $gdb_target_cmd"
+    gdbtk_busy
     set err [catch {gdb_immediate "target $gdb_target_cmd"} msg ]
     $srcWin set_status
+    gdbtk_idle
 
     if {$err} {
       if {[string first "Program not killed" $msg] != -1} {
@@ -1096,7 +1134,7 @@ proc set_target {} {
       set dialog_title "GDB"
       set debugger_name "GDB"
       tk_messageBox -icon error -title $dialog_title -type ok \
-	-modal task -message "$msg\n\n$debugger_name cannot connect to the target board\
+	-message "$msg\n\n$debugger_name cannot connect to the target board\
 using [lindex $gdb_target_cmd 1].\nVerify that the board is securely connected and, if\
 necessary,\nmodify the port setting with the debugger preferences."
       return ERROR
@@ -1122,7 +1160,7 @@ necessary,\nmodify the port setting with the debugger preferences."
 proc run_executable { {auto_start 1} } {
   global gdb_loaded gdb_downloading gdb_target_name
   global gdb_exe_changed gdb_target_changed gdb_program_has_run
-  global gdb_running gdb_exe_name tcl_platform
+  global gdb_running gdb_exe_name gdbtk_platform
 
 #  debug "auto_start=$auto_start gdb_target_name=$gdb_target_name"
 
@@ -1147,7 +1185,8 @@ proc run_executable { {auto_start 1} } {
 
     # Attach
     if {$gdb_target_name == "" || [pref get gdb/src/run_attach]} {
-      if {[gdbtk_attach_remote] == "ATTACH_CANCELED"} {
+      set r [gdbtk_attach_remote]
+      if {$r == "ATTACH_CANCELED" || $r == "ATTACH_ERROR"} {
 	return
       }
     }
@@ -1217,7 +1256,7 @@ proc run_executable { {auto_start 1} } {
 
     # If the user requested it, start an xterm for use as the
     # inferior's tty.
-    if {$tcl_platform(platform) != "windows"
+    if {$gdbtk_platform(platform) != "windows"
 	&& [pref getd gdb/process/xtermtty] == "yes"} {
       tty::create
     }
@@ -1261,7 +1300,7 @@ proc run_executable { {auto_start 1} } {
     } else {
       SrcWin::point_to_main
     }
-
+    
     gdbtk_update
     gdbtk_idle
   } elseif {[pref get gdb/mode]} {
@@ -1457,15 +1496,16 @@ proc gdbtk_detached {} {
 # the debugger must be able to keep gui alive while target_wait is
 # blocking (so that the user can interrupt or detach from it).
 # 
-# The best solution for this is to capture gdb deep down where it
-# can block. For _any_ target board, this will be in either
-# serial or socket code. These places call ui_loop_hook to 
-# keep us alive. For native unix, we use an interval timer.
-# Simulators either call ui_loop_hook directly (older sims, at least)
-# or they call gdb's os_poll_quit callback, where we insert a call
-# to ui_loop_hook. Some targets (like v850ice and windows native)
-# require a call to ui_loop_hook directly in target_wait. See comments
-# before gdb_stop and x_event to find out more about how this is accomplished.
+# The best solution for this is to capture gdb deep down where it can
+# block. For _any_ target board, this will be in either serial or
+# socket code. These places call deprecated_ui_loop_hook to keep us
+# alive. For native unix, we use an interval timer.  Simulators either
+# call deprecated_ui_loop_hook directly (older sims, at least) or they
+# call gdb's os_poll_quit callback, where we insert a call to
+# deprecated_ui_loop_hook. Some targets (like v850ice and windows
+# native) require a call to deprecated_ui_loop_hook directly in
+# target_wait. See comments before gdb_stop and x_event to find out
+# more about how this is accomplished.
 #
 # The stop button's behavior:
 # Pressing the stop button should attempt to stop the target. If, after
@@ -1477,7 +1517,7 @@ proc gdbtk_stop {} {
 
   if {$_gdbtk_stop(timer) == ""} {
     add_hook gdb_idle_hook gdbtk_stop_idle_callback
-    set _gdbtk_stop(timer) [after 3000 gdbtk_detach]
+    set _gdbtk_stop(timer) [after 15000 gdbtk_detach]
     catch {gdb_stop}
   }
 }
@@ -1534,6 +1574,14 @@ proc gdbtk_detach {} {
 #  PROC: gdbtk_run
 # ------------------------------------------------------------------
 proc gdbtk_run {} {
+  if {$::gdb_running == 1} {
+    set msg "A program is currently being debugged.\n"
+    append msg "Do you want to restart?"
+    if {![gdbtk_tcl_query $msg no]} {
+      # NO
+      return
+    }
+  }
   run_executable
 }
 
@@ -1548,7 +1596,7 @@ proc gdbtk_attach_native {} {
     if {[AttachDlg::last_button]} {
 	set pid [AttachDlg::pid]
 	set symbol_file [AttachDlg::symbol_file]
-	if {![_open_file $symbol_file]} {
+	if {$symbol_file != "" && ![_open_file $symbol_file]} {
 	    ManagedWin::open WarningDlg -transient \
 		    -message "Could not load symbols from $symbol_file."
 	    return
@@ -1734,6 +1782,9 @@ proc initialize_gdbtk {} {
     set gdbtk_state(readlineShowUser) 1
   }
 
+  # flush cache for gdbtk_locate_main
+  gdbtk_locate_main 1
+
   # check for existence of a kod command and get it's name and
   # text for menu entry
   set gdb_kod_cmd ""
@@ -1765,3 +1816,14 @@ proc gdbtk_tcl_architecture_changed {} {
   GDBEventHandler::dispatch $e
   delete object $e
 }
+
+proc gdbtk_console_read {} {
+  if {$::gdbtk_state(console) == ""} {
+    ManagedWin::open Console -force
+  } else {
+    raise [namespace tail $::gdbtk_state(console)]
+  }
+  set result [$::gdbtk_state(console) gets]
+  debug "result=$result"
+  return $result
+}
